#include "IniParser.h"


bool operator<( const CIniEntry & ie1, const CIniEntry & ie2 )
{
	return ie1.m_qstrKey < ie2.m_qstrKey;
}

/*
 * parses INI file
 * return true on success, false on failure
 */
bool IniParser::loadIni( const QString & qstrFileName )
{
	m_qlList.clear();

	QFile qfFile( qstrFileName );
	QString qstrSection;

	if( ! qfFile.open( QIODevice::ReadOnly | QIODevice::Text ) )
		return false;

	while ( ! qfFile.atEnd() )
	{
		QString qstrLine = qfFile.readLine().simplified();

		/* we've got comment or empty line, ignore */
		if( qstrLine[ 0 ] == '#' || qstrLine.isEmpty() )
			continue;

		/* we've got section line, section can't be empty */
		if( qstrLine[ 0 ] == '[' )
		{
			qstrLine.remove( '[' );
			qstrLine.remove( ']' );
			qstrSection = qstrLine.simplified();
			if ( qstrSection.isEmpty() )
				goto bail;
			continue;
		}

		/* we've reached "key = val" line, need section at this point */
		if( qstrSection.isEmpty() )
			goto bail;

		/* check the line and clean it */
		if( ! qstrLine.contains('=') )
			goto bail;
		qstrLine.remove(';');

		/* build "key" and add IniEntry */
		CIniEntry ie;
		ie.m_qstrKey = qstrSection;
		ie.m_qstrKey.append( ":" );
		ie.m_qstrKey.append( qstrLine.section( '=', 0, 0 ).simplified() );
		ie.m_qstrVal = qstrLine.section( '=', 1 ).simplified();
		ie.m_bDumpable = true;
		m_qlList.append( ie );
	}

	qSort( m_qlList );
	qfFile.close();
	return true;

bail:
	qWarning( "Error parsing ini, file ignored." );
	m_qlList.clear();
	qfFile.close();
	return false;
}

/*
 * dumps INI list to file
 * return true on success, false on failure
 */
bool IniParser::dumpIni( const QString & qstrFileName, EIniFormat eif ) const
{
	/* don't even try to open a file if "list" is empty */
	if( m_qlList.isEmpty() )
		return false;

	QFile qfFile( qstrFileName );
	QString qstrSection, qstrLastSection;

	if ( ! qfFile.open( QIODevice::WriteOnly | QIODevice::Text ) )
		return false;

	QTextStream qtsOut( &qfFile );

	for( int i = 0; i < m_qlList.size(); i++ )
	{
		qstrSection = m_qlList.at( i ).m_qstrKey.section( ':', 0, 0 );

		/* section appeared/changed, need to write section line */
		if( qstrSection != qstrLastSection )
		{
			if( i != 0 )
				qtsOut << endl;

			qtsOut << '[' << qstrSection << ']' << endl;
			qstrLastSection = qstrSection;
		}

		/* write line, but only if set "dumpable" */
		if( m_qlList.at( i ).m_bDumpable )
		{
			qtsOut << m_qlList.at(i).m_qstrKey.section( ':', 1, 1 );

			if( eif == eifLauncher )
				qtsOut << " = ";
			else
				qtsOut << "=";

			qtsOut << m_qlList.at(i).m_qstrVal;

			if( eif == eifLauncher )
				qtsOut << ';';

			qtsOut << endl;
		}
	}

	qfFile.close();

	return true;
}

/*
 * adds or modifies one entry keeping the list sorted
 * returns true on success, false on improper data
 */
bool IniParser::setEntry( const QString & qstrKey, const QString & qstrVal )
{
	/* check if "key" got ':' in the middle */
	QRegExp rx( ".+:.+" );
	if( ! rx.exactMatch( qstrKey ) )
		return false;

	int i;

	for( i = 0; i < m_qlList.size(); i++ )
		if( m_qlList.at( i ).m_qstrKey == qstrKey )
			break;

	/* nothing found, add new */
	if( i == m_qlList.size() )
	{
		CIniEntry ie;
		ie.m_qstrKey = qstrKey;
		ie.m_qstrVal = qstrVal;
		ie.m_bDumpable = true;

		m_qlList.append( ie );
		qSort( m_qlList );
		return true;
	}

	m_qlList[ i ].m_qstrVal = qstrVal;
	return true;
}

/*
 * deletes entry
 * return true on success, false on entry not found
 */
bool IniParser::delEntry( const QString & qstrKey )
{
	for( int i = 0; i < m_qlList.size(); i++ )
		if( m_qlList.at( i ).m_qstrKey == qstrKey )
		{
			m_qlList.removeAt( i );
			return true;
		}

	return false;
}

/*
 * returns entry value
 * returns empty string on entry not found
 */
QString IniParser::getEntry( const QString & qstrKey ) const
{
	for( int i = 0; i < m_qlList.size(); i++ )
		if( m_qlList.at( i ).m_qstrKey == qstrKey )
			return m_qlList.at( i ).m_qstrVal;

	return QString();
}

/*
 * returns entries list
 */
const QIniEntryList & IniParser::getEntries() const
{
	return m_qlList;
}

/*
 * sets flags of all entries to dumpable
 */
void IniParser::setDumpable( bool bDumpable )
{
	for( int i = 0; i < m_qlList.size(); i++)
		m_qlList[ i ].m_bDumpable = bDumpable;
}

/*
 * sets entry's dumpable flag
 * returns false on entry not found
 */
bool IniParser::setDumpable( const QString & qstrKey, bool bDumpable )
{
	for( int i = 0; i < m_qlList.size(); i++ )
		if( m_qlList.at( i ).m_qstrKey == qstrKey )
		{
			m_qlList[ i ].m_bDumpable = bDumpable;
			return true;
		}

	return false;
}
