#include "TabVideo.h"
#include "ui_TabVideo.h"

#include "System.h"
#include "Registry.h"

TabVideo::TabVideo( QWidget * parent ) :
	QGroupBox( parent ),
	ui( new Ui::TabVideo )
{
	ui->setupUi( this );

	connect( ui -> qsAF, SIGNAL( valueChanged( int ) ), ui -> qlAFValue, SLOT( setValue( int ) ) );
	connect( ui -> qsAA, SIGNAL( valueChanged( int ) ), ui -> qlAAValue, SLOT( setValue( int ) ) );

	ui -> qlAFValue -> setFixedWidth( 20 );
	ui -> qlAAValue -> setFixedWidth( 20 );

	/* fill missing data */
	QStringList qstrlRes = System::enumerateResolutions();
	ui->qcbResolution ->addItems( qstrlRes );

	QList< int > qlBPP = System::enumerateBPP();
	for( int i = 0; i < qlBPP.size(); ++i )
		ui->qcbColorDepth->addItem( QString( "%1-bit" ).arg( qlBPP.at( i ) ) );

	connect( ui->qcbResolution, SIGNAL( currentIndexChanged( const QString& ) ), this, SLOT( setResolution( const QString& ) ) );
	connect( ui->qcbColorDepth, SIGNAL( currentIndexChanged( const QString& ) ), this, SLOT( setColorDepth( const QString& ) ) );
	connect( ui->qcbTextureFilter, SIGNAL( currentIndexChanged( int ) ), this, SLOT( setTextureFilter( int ) ) );
	connect( ui->qsAF, SIGNAL( valueChanged( int ) ), this, SLOT( setAnisotropic( int ) ) );
	connect( ui->qsAA, SIGNAL( valueChanged( int ) ), this, SLOT( setAntialiasing( int ) ) );
	connect( ui->qcbComputerSpeed, SIGNAL( currentIndexChanged( const QString& ) ), this, SLOT( setComputerSpeed( const QString& ) ) );
}

TabVideo::~TabVideo()
{
	delete ui;
}

void TabVideo::updateContent()
{
	/* resolution */
	QString qstrRes = REGISTRY.getResolution();

	/* nothing is set, or the value is not valid for us, so let's set ours */
	int nResIndex = ui->qcbResolution->findText( qstrRes, Qt::MatchExactly );
	if( qstrRes.isEmpty() || nResIndex < 0 )
		setResolution( ui->qcbResolution->currentText() );
	/* otherwise set the value */
	else
		ui->qcbResolution->setCurrentIndex( nResIndex );

	/* color depth */
	int nColorDepth = REGISTRY.getColorDepth();

	/* nothing is set, or the value is not valid for us, so let's set ours */
	int nDepthIndex = ui->qcbColorDepth->findText( QString( "%1" ).arg( nColorDepth ), Qt::MatchStartsWith );
	if( nColorDepth < 0 || nDepthIndex < 0 )
		setColorDepth( ui->qcbColorDepth->currentText() );
	/* otherwise set the value */
	else
		ui->qcbColorDepth->setCurrentIndex( nDepthIndex );

	/* texture filtering */
	int nFilter = REGISTRY.getTextureFilter();

	/* nothing is set, or the value is not valid for us, so let's set ours */
	if( nFilter < 0 || nFilter >= ui->qcbTextureFilter->count() )
		setTextureFilter( ui->qcbTextureFilter->currentIndex() );
	/* otherwise set the value */
	else
		ui->qcbTextureFilter->setCurrentIndex( nFilter );

	/* anisotropic */
	float fAF = REGISTRY.getAnisotropic();

	/* nothing is set, or the value is not valid for us, so let's set ours */
	int nAF = ( int )qMax( fAF, 0.0f );
	// ugly and won't properly resolve all possible values but will the ones set by us
	if( nAF > 2 )
		nAF = ( int )( log( nAF ) / log( 2 ) ) + 1;

	if( fAF < 0 || nAF > ui->qsAF->maximum() )
		setAnisotropic( ui->qsAF->value() );
	/* otherwise set the value */
	else
		ui->qsAF->setValue( nAF );

	/* antialiasing */
	int nAA = REGISTRY.getAntialiasing();

	/* nothing is set, or the value is not valid for us, so let's set ours */
	if( nAA < 0 || nAA > ui->qsAA->maximum() )
		setAntialiasing( ui->qsAA->value() );
	/* otherwise set the value */
	else
		ui->qsAA->setValue( nAA );

	/* computer speed */
	int nComputerSpeed = REGISTRY.getComputerSpeed();

	/* nothing is set, or the value is not valid for us, so let's set ours */
	int nSpeedIndex = ui->qcbComputerSpeed->findText( QString( "%1" ).arg( nComputerSpeed ), Qt::MatchStartsWith );
	if( nComputerSpeed < 0 || nSpeedIndex < 0 )
		setComputerSpeed( ui->qcbComputerSpeed->currentText() );
	/* otherwise set the value */
	else
		ui->qcbComputerSpeed->setCurrentIndex( nSpeedIndex );
}

void TabVideo::setResolution( const QString & qstrRes )
{
	REGISTRY.setResolution( qstrRes );
	emit setRegistryChanged();
}

void TabVideo::setColorDepth( const QString & qstrDepth )
{
	REGISTRY.setColorDepth( qstrDepth.section( '-', 0, 0 ).toInt() );
	emit setRegistryChanged();
}

void TabVideo::setTextureFilter( const int & nFilter )
{
	REGISTRY.setTextureFilter( nFilter );
	emit setRegistryChanged();
}

void TabVideo::setAnisotropic( const int & nAF )
{
	float fAF;

	switch( nAF )
	{
	case 1:
		fAF = 1.0f;
		break;
	case 2:
		fAF = 2.0f;
		break;
	case 3:
		fAF = 4.0f;
		break;
	case 4:
		fAF = 8.0f;
		break;
	case 5:
		fAF = 16.0f;
		break;
	default:
		fAF = 0.0f;
	}

	REGISTRY.setAnisotropic( fAF );
	emit setRegistryChanged();
}

void TabVideo::setAntialiasing( const int & nAA )
{
	REGISTRY.setAntialiasing( nAA );
	emit setRegistryChanged();
}

void TabVideo::setComputerSpeed( const QString & qstrSpeed )
{
	REGISTRY.setComputerSpeed( qstrSpeed.section( '.', 0, 0 ).toInt() );
	emit setRegistryChanged();
}
